@@ -30,10 +30,11 @@
 // complete_add function only accepts one short switch and one long switch.
 
 /// Silly function.
-static void builtin_complete_add2(const wchar_t *cmd, bool cmd_is_path, const wchar_t *short_opt,
+static void builtin_complete_add2(const wcstring &cmd, bool cmd_is_path, const wchar_t *short_opt,
                                   const wcstring_list_t &gnu_opts, const wcstring_list_t &old_opts,
-                                  completion_mode_t result_mode, const wchar_t *condition,
-                                  const wchar_t *comp, const wchar_t *desc, int flags) {
+                                  completion_mode_t result_mode, const wcstring_list_t &condition,
+                                  const wchar_t *comp, const wchar_t *desc,
+                                  complete_flags_t flags) {
     for (const wchar_t *s = short_opt; *s; s++) {
         complete_add(cmd, cmd_is_path, wcstring{*s}, option_type_short, result_mode, condition,
                      comp, desc, flags);
@@ -59,16 +60,16 @@ static void builtin_complete_add2(const wchar_t *cmd, bool cmd_is_path, const wc
 static void builtin_complete_add(const wcstring_list_t &cmds, const wcstring_list_t &paths,
                                  const wchar_t *short_opt, const wcstring_list_t &gnu_opt,
                                  const wcstring_list_t &old_opt, completion_mode_t result_mode,
-                                 const wchar_t *condition, const wchar_t *comp, const wchar_t *desc,
-                                 int flags) {
+                                 const wcstring_list_t &condition, const wchar_t *comp,
+                                 const wchar_t *desc, complete_flags_t flags) {
     for (const wcstring &cmd : cmds) {
-        builtin_complete_add2(cmd.c_str(), false /* not path */, short_opt, gnu_opt, old_opt,
-                              result_mode, condition, comp, desc, flags);
+        builtin_complete_add2(cmd, false /* not path */, short_opt, gnu_opt, old_opt, result_mode,
+                              condition, comp, desc, flags);
     }
 
     for (const wcstring &path : paths) {
-        builtin_complete_add2(path.c_str(), true /* is path */, short_opt, gnu_opt, old_opt,
-                              result_mode, condition, comp, desc, flags);
+        builtin_complete_add2(path, true /* is path */, short_opt, gnu_opt, old_opt, result_mode,
+                              condition, comp, desc, flags);
     }
 }
 
@@ -135,7 +136,8 @@ maybe_t<int> builtin_complete(parser_t &parser, io_streams_t &streams, const wch
     int remove = 0;
     wcstring short_opt;
     wcstring_list_t gnu_opt, old_opt, subcommand;
-    const wchar_t *comp = L"", *desc = L"", *condition = L"";
+    const wchar_t *comp = L"", *desc = L"";
+    wcstring_list_t condition;
     bool do_complete = false;
     bool have_do_complete_param = false;
     wcstring do_complete_param;
@@ -168,7 +170,9 @@ maybe_t<int> builtin_complete(parser_t &parser, io_streams_t &streams, const wch
         {L"help", no_argument, nullptr, 'h'},
         {L"keep-order", no_argument, nullptr, 'k'},
         {L"escape", no_argument, nullptr, opt_escape},
-        {nullptr, 0, nullptr, 0}};
+        {}};
+
+    bool have_x = false;
 
     int opt;
     wgetopter_t w;
@@ -177,6 +181,8 @@ maybe_t<int> builtin_complete(parser_t &parser, io_streams_t &streams, const wch
             case 'x': {
                 result_mode.no_files = true;
                 result_mode.requires_param = true;
+                // Needed to print an error later.
+                have_x = true;
                 break;
             }
             case 'f': {
@@ -264,8 +270,8 @@ maybe_t<int> builtin_complete(parser_t &parser, io_streams_t &streams, const wch
                 break;
             }
             case 'n': {
-                condition = w.woptarg;
-                assert(condition);
+                condition.push_back(w.woptarg);
+                assert(w.woptarg);
                 break;
             }
             case 'w': {
@@ -301,8 +307,15 @@ maybe_t<int> builtin_complete(parser_t &parser, io_streams_t &streams, const wch
     }
 
     if (result_mode.no_files && result_mode.force_files) {
-        streams.err.append_format(BUILTIN_ERR_COMBO2, L"complete",
-                                  L"'--no-files' and '--force-files'");
+        if (!have_x) {
+            streams.err.append_format(BUILTIN_ERR_COMBO2, L"complete",
+                                      L"'--no-files' and '--force-files'");
+        } else {
+            // The reason for us not wanting files is `-x`,
+            // which is short for `-rf`.
+            streams.err.append_format(BUILTIN_ERR_COMBO2, L"complete",
+                                      L"'--exclusive' and '--force-files'");
+        }
         return STATUS_INVALID_ARGS;
     }
 
@@ -322,12 +335,11 @@ maybe_t<int> builtin_complete(parser_t &parser, io_streams_t &streams, const wch
         }
     }
 
-    if (condition && std::wcslen(condition)) {
-        const wcstring condition_string = condition;
+    for (const auto &condition_string : condition) {
         parse_error_list_t errors;
         if (parse_util_detect_errors(condition_string, &errors)) {
             streams.err.append_format(L"%ls: Condition '%ls' contained a syntax error", cmd,
-                                      condition);
+                                      condition_string.c_str());
             for (const auto &error : errors) {
                 streams.err.append_format(L"\n%ls: ", cmd);
                 streams.err.append(error.describe(condition_string, parser.is_interactive()));
@@ -336,7 +348,7 @@ maybe_t<int> builtin_complete(parser_t &parser, io_streams_t &streams, const wch
         }
     }
 
-    if (comp && std::wcslen(comp)) {
+    if (comp && *comp) {
         wcstring prefix;
         prefix.append(cmd);
         prefix.append(L": ");
@@ -417,8 +429,8 @@ maybe_t<int> builtin_complete(parser_t &parser, io_streams_t &streams, const wch
             parser.libdata().builtin_complete_current_commandline = false;
         }
     } else if (path.empty() && gnu_opt.empty() && short_opt.empty() && old_opt.empty() && !remove &&
-               !*comp && !*desc && !*condition && wrap_targets.empty() && !result_mode.no_files &&
-               !result_mode.force_files && !result_mode.requires_param) {
+               !*comp && !*desc && condition.empty() && wrap_targets.empty() &&
+               !result_mode.no_files && !result_mode.force_files && !result_mode.requires_param) {
         // No arguments that would add or remove anything specified, so we print the definitions of
         // all matching completions.
         if (cmd_to_complete.empty()) {
